@@ -48,6 +48,10 @@ struct _GdmConnection {
 
 	GString *buffer;
 
+	int message_count;
+
+	gboolean nonblock;
+
 	int close_level; /* 0 - normal
 			    1 - no close, when called raise to 2
 			    2 - close was requested */
@@ -73,7 +77,7 @@ static gboolean
 close_if_needed (GdmConnection *conn, GIOCondition cond)
 {
 	/* non-subconnections are never closed */
-	if (conn->filename != NULL)
+	if (conn->parent == NULL)
 		return TRUE;
 
 	if (cond & G_IO_ERR ||
@@ -115,8 +119,11 @@ gdm_connection_handler (GIOChannel *source,
 		     ve_string_empty (conn->buffer->str)))
 			/*ignore \r or empty lines*/
 			continue;
-		if (*p == '\n') {
+		if (*p == '\n' || 
+		    /* cut lines short at 4096 to prevent DoS attacks */
+		    conn->buffer->len > 4096) {
 			conn->close_level = 1;
+			conn->message_count ++;
 			conn->handler (conn, conn->buffer->str,
 				       conn->data);
 			if (conn->close_level == 2) {
@@ -147,6 +154,7 @@ gboolean
 gdm_connection_write (GdmConnection *conn, const char *str)
 {
 	int ret;
+	int flags = 0;
 #ifndef MSG_NOSIGNAL
 	void (*old_handler)(int);
 #endif
@@ -157,11 +165,16 @@ gdm_connection_write (GdmConnection *conn, const char *str)
 	if ( ! conn->writable)
 		return FALSE;
 
+#ifdef MSG_DONTWAIT
+	if (conn->nonblock)
+		flags |= MSG_DONTWAIT;
+#endif
+
 #ifdef MSG_NOSIGNAL
-	ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL);
+	ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL | flags);
 #else
 	old_handler = signal (SIGPIPE, SIG_IGN);
-	ret = send (conn->fd, str, strlen (str), 0);
+	ret = send (conn->fd, str, strlen (str), flags);
 	signal (SIGPIPE, old_handler);
 #endif
 
@@ -197,6 +210,8 @@ gdm_socket_handler (GIOChannel *source,
 	gdm_debug ("gdm_socket_handler: Accepting new connection fd %d", fd);
 
 	newconn = g_new0 (GdmConnection, 1);
+	newconn->message_count = 0;
+	newconn->nonblock = conn->nonblock;
 	newconn->close_level = 0;
 	newconn->fd = fd;
 	newconn->writable = TRUE;
@@ -221,6 +236,7 @@ gdm_socket_handler (GIOChannel *source,
 	}
 
 	unixchan = g_io_channel_unix_new (newconn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
 	g_io_channel_set_buffered (unixchan, FALSE);
 
 	newconn->source = g_io_add_watch_full
@@ -263,6 +279,8 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 	chmod (sockname, mode);
 
 	conn = g_new0 (GdmConnection, 1);
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
 	conn->close_level = 0;
 	conn->fd = fd;
 	conn->writable = FALSE;
@@ -274,6 +292,43 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 	conn->n_subconnections = 0;
 
 	unixchan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
+	g_io_channel_set_buffered (unixchan, FALSE);
+
+	conn->source = g_io_add_watch_full
+		(unixchan, G_PRIORITY_DEFAULT,
+		 G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		 gdm_socket_handler, conn, NULL);
+	g_io_channel_unref (unixchan);
+
+	listen (fd, 5);
+
+	return conn;
+}
+
+GdmConnection *
+gdm_connection_open_fd (int fd)
+{
+	GIOChannel *unixchan;
+	GdmConnection *conn;
+
+	g_return_val_if_fail (fd < 0, NULL);
+
+	conn = g_new0 (GdmConnection, 1);
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
+	conn->close_level = 0;
+	conn->fd = fd;
+	conn->writable = FALSE;
+	conn->buffer = NULL;
+	conn->filename = NULL;
+	conn->user_flags = 0;
+	conn->parent = NULL;
+	conn->subconnections = NULL;
+	conn->n_subconnections = 0;
+
+	unixchan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
 	g_io_channel_set_buffered (unixchan, FALSE);
 
 	conn->source = g_io_add_watch_full
@@ -313,6 +368,8 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	chmod (fifo, mode);
 
 	conn = g_new0 (GdmConnection, 1);
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
 	conn->close_level = 0;
 	conn->fd = fd;
 	conn->writable = FALSE;
@@ -324,6 +381,7 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	conn->n_subconnections = 0;
 
 	fifochan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (fifochan, NULL, NULL);
 	g_io_channel_set_buffered (fifochan, FALSE);
 
 	conn->source = g_io_add_watch_full
@@ -442,4 +500,44 @@ gdm_connection_set_close_notify (GdmConnection *conn,
 	conn->close_notify = close_notify;
 }
 
+gboolean 
+gdm_connection_printf (GdmConnection *conn, const gchar *format, ...)
+{
+	va_list args;
+	gboolean ret;
+	gchar *s;
+
+	va_start (args, format);
+	s = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	ret = gdm_connection_write (conn, s);
+
+	g_free (s);
+
+	return ret;
+}
+
+int
+gdm_connection_get_message_count (GdmConnection *conn)
+{
+	g_return_val_if_fail (conn != NULL, -1);
+	return conn->message_count;
+}
+
+gboolean
+gdm_connection_get_nonblock (GdmConnection *conn)
+{
+	g_return_val_if_fail (conn != NULL, FALSE);
+	return conn->nonblock;
+}
+
+void
+gdm_connection_set_nonblock (GdmConnection *conn,
+			     gboolean nonblock)
+{
+	g_return_if_fail (conn != NULL);
+	conn->nonblock = nonblock;
+}
+
 /* EOF */
