@@ -752,27 +752,42 @@ void
 gdm_ensure_sanity (void)
 {
 	mode_t old_umask;
+	uid_t old_euid;
+	gid_t old_egid;
+
+	old_euid = geteuid ();
+	old_egid = getegid ();
+
+	seteuid (0);
+	setegid (0);
 
 	/* The /tmp/.ICE-unix check, note that we do
 	 * ignore errors, since it's not deadly to run
 	 * if we can't perform this task :) */
 	old_umask = umask (0);
 
-        if (mkdir ("/tmp/.ICE-unix", 0777) == 0) {
-		/* Make sure it is root */
-		if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
-			chmod ("/tmp/.ICE-unix", 01777);
-        } else {
+        if (mkdir ("/tmp/.ICE-unix", 01777) != 0) {
+		/* if we can't create it, perhaps it
+		   already exists, in which case ensure the
+		   correct permissions */
 		struct stat s;
-		if (lstat ("/tmp/.ICE-unix", &s) == 0 &&
-		    S_ISDIR (s.st_mode)) {
+		int r;
+		IGNORE_EINTR (r = lstat ("/tmp/.ICE-unix", &s));
+		if (r == 0 && S_ISDIR (s.st_mode)) {
 			/* Make sure it is root and sticky */
-			if (chown ("/tmp/.ICE-unix", 0, 0) == 0)
-				chmod ("/tmp/.ICE-unix", 01777);
+			IGNORE_EINTR (chown ("/tmp/.ICE-unix", 0, 0));
+			IGNORE_EINTR (chmod ("/tmp/.ICE-unix", 01777));
+		} else {
+			/* There is a file/link/whatever called .ICE-unix? whack and try mkdir */
+			unlink ("/tmp/.ICE-unix");
+			mkdir ("/tmp/.ICE-unix", 01777);
 		}
 	}
 
 	umask (old_umask);
+
+	seteuid (old_euid);
+	setegid (old_egid);
 }
 
 const GList *
