@@ -119,11 +119,11 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
-	    sleep (GdmRetryDelay);
+	    gdm_sleep_no_signal (GdmRetryDelay);
 	    gdm_error (_("Couldn't authenticate user"));
 	    /* FIXME: Hmm, how are we sure that the login is username
 	     * and password.  That is the most common case but not
-	     * neccessairly true, this message needs to be changed
+	     * necessarily true, this message needs to be changed
 	     * to allow for such cases */
 	    auth_errmsg = g_strdup_printf
 		    (_("\nIncorrect username or password. "
@@ -140,10 +140,10 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    sleep (GdmRetryDelay);
+	    gdm_sleep_no_signal (GdmRetryDelay);
 	    /* FIXME: Hmm, how are we sure that the login is username
 	     * and password.  That is the most common case but not
-	     * neccessairly true, this message needs to be changed
+	     * necessarily true, this message needs to be changed
 	     * to allow for such cases */
 	    auth_errmsg = g_strdup_printf
 		    (_("\nIncorrect username or password. "
