@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -39,6 +40,17 @@
 #include "gdm.h"
 #include "gdmcomm.h"
 
+static pid_t xnest_pid = 0;
+
+static void
+term_handler (int sig)
+{
+	if (xnest_pid != 0)
+		kill (xnest_pid, SIGTERM);
+	else
+		exit (0);
+}
+
 static int
 get_free_display (void)
 {
@@ -52,6 +64,8 @@ get_free_display (void)
 	for (i = 20; i < 3000; i ++) {
 		struct stat s;
 		char buf[256];
+		FILE *fp;
+
 		sock = socket (AF_INET, SOCK_STREAM, 0);
 
 		serv_addr.sin_port = htons (6000 + i);
@@ -66,6 +80,23 @@ get_free_display (void)
 
 		close (sock);
 
+		/* if lock file exists and the process exists */
+		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
+		fp = fopen (buf, "r");
+		if (fp != NULL) {
+			char buf2[100];
+			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
+				gulong pid;
+				if (sscanf (buf2, "%lu", &pid) == 1 &&
+				    kill (pid, 0) == 0) {
+					fclose (fp);
+					continue;
+				}
+
+			}
+			fclose (fp);
+		}
+
 		g_snprintf (buf, sizeof (buf), "/tmp/.X11-unix/X%d", i);
 		if (stat (buf, &s) == 0 &&
 		    s.st_uid != getuid ()) {
@@ -403,7 +434,7 @@ main (int argc, char *argv[])
 	const char **args;
 	char *xnest;
 	char **execvec;
-	pid_t pid;
+	struct sigaction term;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -533,6 +564,7 @@ main (int argc, char *argv[])
 	}
 
 	printf ("DISPLAY=:%d\n", display);
+	fflush (stdout);
 
 	g_snprintf (display_num, sizeof (display_num), ":%d", display);
 	g_snprintf (indirect_host, sizeof (indirect_host), "%s", host);
@@ -546,10 +578,24 @@ main (int argc, char *argv[])
 		if (fork () > 0) {
 			_exit (0);
 		}
+		close (0);
+		close (1);
+		close (2);
+		open ("/dev/null", O_RDWR);
+		open ("/dev/null", O_RDONLY);
+		open ("/dev/null", O_RDONLY);
 	}
 
-	pid = fork ();
-	if (pid == 0) {
+	term.sa_handler = term_handler;
+	term.sa_flags = SA_RESTART;
+	sigemptyset (&term.sa_mask);
+
+	sigaction (SIGTERM, &term, NULL);
+	sigaction (SIGINT, &term, NULL);
+	sigaction (SIGHUP, &term, NULL);
+
+	xnest_pid = fork ();
+	if (xnest_pid == 0) {
 		execvp (execvec[0], execvec);
 		g_warning ("Can't exec, trying Xnest");
 		execvec[0] = "Xnest";
@@ -557,13 +603,14 @@ main (int argc, char *argv[])
 		g_warning ("Can't exec that either, giving up");
 		/* FIXME: this should be handled in the GUI */
 		_exit (1);
-	} else if (pid < 0) {
+	} else if (xnest_pid < 0) {
 		/* eeeek */
 		g_warning ("Can't fork");
 		_exit (1);
 	}
 
-	ve_waitpid_no_signal (pid, 0, 0);
+	ve_waitpid_no_signal (xnest_pid, 0, 0);
+	xnest_pid = 0;
 
 	socket = g_strdup_printf ("/tmp/.X11-unix/X%d", display);
 	unlink (socket);
