@@ -63,7 +63,8 @@ gdm_check (void)
 			 GTK_MESSAGE_WARNING,
 			 GTK_BUTTONS_OK,
 			 "foo");
-		 gtk_label_set_markup
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		gtk_label_set_markup
 			 (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
 			  _("<b>GDM (The GNOME Display Manager) "
 			    "is not running.</b>\n\n"
@@ -131,6 +132,7 @@ main (int argc, char *argv[])
 					      "system administrator to enable "
 					      "it\nin the GDM configurator "
 					      "program."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 	}
@@ -183,6 +185,7 @@ main (int argc, char *argv[])
 						    GTK_MESSAGE_WARNING,
 						    GTK_BUTTONS_OK,
 						    _("No picture selected."));
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
 		} else if (s.st_size > max_size) {
@@ -197,6 +200,7 @@ main (int argc, char *argv[])
 						      "then %d bytes to\n"
 						      "show in the face browser"),
 						    max_size);
+			gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (d);
 		} else {
@@ -221,6 +225,7 @@ main (int argc, char *argv[])
 							      "reading\nError: %s"),
 							    pixmap,
 							    g_strerror (errno));
+				gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
 				g_free (cfg_file);
@@ -239,6 +244,7 @@ main (int argc, char *argv[])
 							      "writing\nError: %s"),
 							    photofile,
 							    g_strerror (errno));
+				gtk_dialog_set_has_separator (GTK_DIALOG (d), FALSE);
 				gtk_dialog_run (GTK_DIALOG (d));
 				gtk_widget_destroy (d);
 				g_free (cfg_file);
