@@ -60,6 +60,7 @@ static gboolean used_defaults = FALSE;
 gint greeter_current_delay = 0;
 
 extern gboolean session_dir_whacked_out;
+extern gboolean require_quarter;
 
 
 static void 
@@ -141,8 +142,8 @@ greeter_parse_config (void)
       GdmXineramaScreen = 0;
 }
 
-static void
-setup_cursor (GdkCursorType type)
+void
+greeter_setup_cursor (GdkCursorType type)
 {
 	GdkCursor *cursor = gdk_cursor_new (type);
 	gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
@@ -258,6 +259,7 @@ greeter_ctrl_handler (GIOChannel *source,
 				      GTK_BUTTONS_OK,
 				      "%s",
 				      tmp);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 	g_free (tmp);
 
 	gdm_wm_center_window (GTK_WINDOW (dlg));
@@ -342,6 +344,28 @@ greeter_ctrl_handler (GIOChannel *source,
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
 	greeter_item_timed_stop ();
+
+	if (require_quarter) {
+		/* we should be now fine for focusing new windows */
+		gdm_wm_focus_new_windows (TRUE);
+
+		dlg = gtk_message_dialog_new (NULL /* parent */,
+					      GTK_DIALOG_MODAL /* flags */,
+					      GTK_MESSAGE_INFO,
+					      GTK_BUTTONS_OK,
+					      /* translators:  This is a nice and evil eggie text, translate
+					       * to your favourite currency */
+					      _("Please insert 25 cents "
+						"to log in."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+		gdm_wm_center_window (GTK_WINDOW (dlg));
+
+		gdm_wm_no_login_focus_push ();
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		gdm_wm_no_login_focus_pop ();
+	}
+
 	greeter_item_pam_leftover_messages ();
 
 	gdk_flush ();
@@ -475,7 +499,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
 	/* Set busy cursor */
-	setup_cursor (GDK_WATCH);
+	greeter_setup_cursor (GDK_WATCH);
 
 	gdm_wm_save_wm_order ();
 
@@ -558,11 +582,12 @@ verify_gdm_version (void)
 					 "You have probably just upgraded gdm.\n"
 					 "Please restart the gdm daemon or reboot the computer."),
 				       VERSION);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
     
@@ -587,6 +612,7 @@ verify_gdm_version (void)
 					 "You have probably just upgraded gdm.\n"
 					 "Please restart the gdm daemon or reboot the computer."),
 				       VERSION);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Reboot"),
 			      RESPONSE_REBOOT,
@@ -597,7 +623,7 @@ verify_gdm_version (void)
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
       
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
@@ -630,6 +656,7 @@ verify_gdm_version (void)
 					 "You have probably just upgraded gdm.\n"
 					 "Please restart the gdm daemon or reboot the computer."),
 				       VERSION, gdm_version);
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Restart"),
 			      RESPONSE_RESTART,
@@ -645,7 +672,7 @@ verify_gdm_version (void)
     
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), RESPONSE_RESTART);
       
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 	{
@@ -678,6 +705,7 @@ greeter_message (const gchar *msg)
 				GTK_BUTTONS_CLOSE,
 				"%s",
 				msg);
+  gtk_dialog_set_has_separator (GTK_DIALOG (req), FALSE);
   g_signal_connect (G_OBJECT (req), "destroy",
 		    G_CALLBACK (gtk_widget_destroyed),
 		    &req);
@@ -708,6 +736,7 @@ greeter_query (const gchar *msg)
 				      GTK_BUTTONS_YES_NO,
 				      "%s",
 				      msg);
+	gtk_dialog_set_has_separator (GTK_DIALOG (req), FALSE);
 	gtk_label_set_use_markup
 		(GTK_LABEL (GTK_MESSAGE_DIALOG (req)->label), TRUE);
 
@@ -866,7 +895,7 @@ greeter_reread_config (int sig, gpointer data)
 			"theme '%s' theme_dir '%s'",
 			theme, theme_dir);
 		/* Set busy cursor */
-		setup_cursor (GDK_WATCH);
+		greeter_setup_cursor (GDK_WATCH);
 
 		gdm_wm_save_wm_order ();
 
@@ -965,6 +994,42 @@ get_theme_file (const char *in, char **theme_dir)
   return file;
 }
 
+/* Not to look too shaby on Xinerama setups */
+static void
+setup_background_color (void)
+{
+  GdkColormap *colormap;
+  GdkColor color;
+  VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+  char *bg_color = ve_config_get_string (config, GDM_KEY_BACKGROUNDCOLOR);
+
+  if (bg_color == NULL ||
+      bg_color[0] == '\0' ||
+      ! gdk_color_parse (bg_color, &color))
+    {
+      gdk_color_parse ("#007777", &color);
+    }
+
+  g_free (bg_color);
+
+  colormap = gdk_drawable_get_colormap
+	  (gdk_get_default_root_window ());
+  /* paranoia */
+  if (colormap != NULL)
+    {
+      gboolean success;
+      gdk_error_trap_push ();
+
+      gdk_colormap_alloc_colors (colormap, &color, 1,
+				 FALSE, TRUE, &success);
+      gdk_window_set_background (gdk_get_default_root_window (), &color);
+      gdk_window_clear (gdk_get_default_root_window ());
+
+      gdk_flush ();
+      gdk_error_trap_pop ();
+    }
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -994,7 +1059,7 @@ main (int argc, char *argv[])
   gtk_init (&argc, &argv);
 
   /* Should be a watch already, but anyway */
-  setup_cursor (GDK_WATCH);
+  greeter_setup_cursor (GDK_WATCH);
 
   if ( ! ve_string_empty (GdmGtkRC))
     gtk_rc_parse (GdmGtkRC);
@@ -1089,12 +1154,31 @@ main (int argc, char *argv[])
 			gdm_wm_screen.height,
 			&error);
 
-    /* FIXME: beter information should be printed */
     if (root == NULL &&
 	g_getenv ("GDM_THEME") != NULL &&
 	DOING_GDM_DEVELOPMENT)
       {
-        g_warning ("Theme could not be loaded");
+        GtkWidget *dialog;
+
+        gdm_wm_init (0);
+        gdm_wm_focus_new_windows (TRUE);
+    
+        dialog = gtk_message_dialog_new (NULL /* parent */,
+				         GTK_DIALOG_MODAL /* flags */,
+				         GTK_MESSAGE_ERROR,
+				         GTK_BUTTONS_OK,
+				         _("There was an error loading the "
+					   "theme %s"),
+					 g_getenv ("GDM_THEME"));
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+    
+        gtk_widget_show_all (dialog);
+        gdm_wm_center_window (GTK_WINDOW (dialog));
+
+        greeter_setup_cursor (GDK_LEFT_PTR);
+    
+        gtk_dialog_run (GTK_DIALOG (dialog));
+
         exit(1);
       }
 
@@ -1111,9 +1195,32 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
-  /* FIXME: beter information should be printed */
   if (greeter_lookup_id ("user-pw-entry") == NULL)
-    root = NULL;
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_init (0);
+      gdm_wm_focus_new_windows (TRUE);
+    
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("The theme for the graphical greeter "
+					 "is corrupt.  It does not contain "
+					 "definition for the username/password "
+					 "entry element."));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      greeter_setup_cursor (GDK_LEFT_PTR);
+
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      root = NULL;
+    }
 
   /* FIXME: beter information should be printed */
   if (DOING_GDM_DEVELOPMENT && root == NULL)
@@ -1138,11 +1245,12 @@ main (int argc, char *argv[])
 					 "also could not have been loaded, "
 					 "I will attempt to start the "
 					 "standard greeter"));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1158,11 +1266,12 @@ main (int argc, char *argv[])
 					 "and you may have to login another "
 					 "way and fix the installation of "
 					 "gdm"));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
     
       gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1177,6 +1286,8 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
+  setup_background_color ();
+
   /* can it ever happen that it'd be NULL here ??? */
   if (window->window != NULL)
     {
@@ -1201,10 +1312,11 @@ main (int argc, char *argv[])
 				       _("Your session directory is missing or empty!\n\n"
 					 "There are two available sessions you can use, but\n"
 					 "you should log in and correct the gdm configuration."));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1225,10 +1337,11 @@ main (int argc, char *argv[])
 				       _("The configuration file contains an invalid command\n"
 					 "line for the login dialog, and thus I ran the\n"
 					 "default command.  Please fix your configuration."));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1251,10 +1364,11 @@ main (int argc, char *argv[])
 					 "defaults to run this session.  You should log in\n"
 					 "and create a configuration file with the GDM\n"
 					 "configuration program."));
+      gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
 
-      setup_cursor (GDK_LEFT_PTR);
+      greeter_setup_cursor (GDK_LEFT_PTR);
 
       gdm_wm_no_login_focus_push ();
       gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1264,7 +1378,7 @@ main (int argc, char *argv[])
 
   gdm_wm_restore_wm_order ();
 
-  setup_cursor (GDK_LEFT_PTR);
+  greeter_setup_cursor (GDK_LEFT_PTR);
 
   gtk_main ();
 
